#!/bin/bash
### BEGIN INIT INFO
# Provides:          litecoin-init.sh
# Required-Start:    networking
# Required-Stop:     networking
# Should-Start:      sshd tor
# Should-Stop:       sshd tor
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: litecoind init script
# Description:       This file should be used to construct scripts to be
#                    placed in /etc/init.d.
### END INIT INFO

PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="Init litecoind"
NAME=litecoin-init.sh
DAEMON=/usr/lib/litecoin-init/litecoind
LITECOIN_CLI=/usr/lib/litecoin-init/litecoin-cli
DAEMON_ARGS=""
SCRIPTNAME=/etc/init.d/$(/usr/bin/basename $0)
STATIC_DEFAULT_CONF=/usr/lib/litecoin-init/default/litecoin-init
DEFAULT_CONF=/etc/default/litecoin-init
LOG_DIR=/var/log
RUN_DIR=/var/lib/litecoin-init

. /lib/init/vars.sh
. /lib/lsb/init-functions

# Exit if the package is not installed
[ -x "$DAEMON" ] || exit 0

if [ -f ${STATIC_DEFAULT_CONF} ]
then
    . ${STATIC_DEFAULT_CONF}
fi

if [ -f ${DEFAULT_CONF} ]
then
    . ${DEFAULT_CONF}
fi

set -e

LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} ).log
TESTNET_LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-testnet.log
REGTEST_LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-regtest.log

LOG_FILE_DEBUG=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-debug.log
TESTNET_LOG_FILE_DEBUG=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-testnet-debug.log
REGTEST_LOG_FILE_DEBUG=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-regtest-debug.log

if [ -z "${RUN_VERSION}" ]
then
    RUN_VERSION=main
fi

if ! [ "${ENABLE}" = "yes" ]
then
    log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
    exit 0
fi

#
# Function that starts the daemon/service
#
do_start()
{
    INIT_TYPE="$1"
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	/bin/echo -n "" >> ${LOG_FILE}
	/bin/echo -n "" >> ${TESTNET_LOG_FILE}
	/bin/echo -n "" >> ${REGTEST_LOG_FILE}
	/bin/echo -n "" >> ${LOG_FILE_DEBUG}
	/bin/echo -n "" >> ${TESTNET_LOG_FILE_DEBUG}
	/bin/echo -n "" >> ${REGTEST_LOG_FILE_DEBUG}

	/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${REGTEST_LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE_DEBUG}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE_DEBUG}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${REGTEST_LOG_FILE_DEBUG}

	/bin/chmod 0640 ${LOG_FILE}
	/bin/chmod 0640 ${TESTNET_LOG_FILE}
	/bin/chmod 0640 ${REGTEST_LOG_FILE}
	/bin/chmod 0640 ${LOG_FILE_DEBUG}
	/bin/chmod 0640 ${TESTNET_LOG_FILE_DEBUG}
	/bin/chmod 0640 ${REGTEST_LOG_FILE_DEBUG}

	case "${RUN_VERSION}" in
	    main)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
	    ;;
	    testnet)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$?
	    ;;
	    both)
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -conf=${SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${LOG_FILE}"
		RESULT=$?
		/bin/su ${RUNUSER} -c "/usr/bin/nice -n ${NICE_VAL} ${DAEMON} -conf=${TESTNET_SERVER_CONF} ${INIT_TYPE} 1>/dev/null 2>>${TESTNET_LOG_FILE}"
		RESULT=$[ ${RESULT} + $? ]
	    ;;
	esac
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

do_start_reindex()
{
    RESULT=0
    do_start -reindex
    RESULT=$?
    return ${RESULT}
}

do_start_rescan()
{
    RESULT=0
    do_start -rescan
    RESULT=$?
    return ${RESULT}
}

do_start_zapwallettxes()
{
    RESULT=0

    MODEZAP=2

    if [ -n "$2" ]
    then
	MODEZAP=$2
    fi

    do_start -zapwallettxes=${MODEZAP}
    RESULT=$?
    return ${RESULT}
}

do_start_upgradewallet()
{
    RESULT=0
    do_start -upgradewallet
    RESULT=$?
    return ${RESULT}
}

#
# Function that stops the daemon/service
#
do_stop()
{
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		PIDO=`/bin/cat ${RUN_DIR}/main/litecoind.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LITECOIN_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
	    ;;
	    testnet)
		TESTNET_PIDO=`/bin/cat ${RUN_DIR}/testnet/litecoind-testnet.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LITECOIN_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
	    ;;
	    both)
		PIDO=`/bin/cat ${RUN_DIR}/main/litecoind.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LITECOIN_CLI} -conf=${SERVER_CONF} stop 1>/dev/null 2>>${LOG_FILE}" || /bin/kill ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		TESTNET_PIDO=`/bin/cat ${RUN_DIR}/testnet/litecoind-testnet.pid 2>/dev/null`
		/bin/su ${RUNUSER} -c "${LITECOIN_CLI} -conf=${TESTNET_SERVER_CONF} stop 1>/dev/null 2>>${TESTNET_LOG_FILE}" || /bin/kill ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$[ ${RESULT} + $? ]
	    ;;
	esac

	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=0
	while [ ${COUNT_PROC} -gt 0 ]
	do
	    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	    COUNTER=$[ ${COUNTER} + 1 ]
	    if [ ${COUNTER} -ge 95 ]
	    then
		COUNT_PROC=0
	    fi
	    /bin/sleep 1
	done
    else
	log_action_msg "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that sends a SIGHUP to the daemon/service
#
do_reload()
{
    do_stop
    /bin/sleep 1
    do_start
}

RESULT=0

case "$1" in
  start)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start litecoind"
	do_start
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-reindex)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start litecoind reindex mode"
	do_start_reindex
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-rescan)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start litecoind rescan mode"
	do_start_rescan
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-zapwallettxes)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start litecoind zapwallettxes mode"
	do_start_zapwallettxes
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  start-upgradewallet)
	log_daemon_msg "${NAME}"
	log_progress_msg "Start litecoind upgradewallet mode"
	do_start_upgradewallet
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  stop)
	log_daemon_msg "${NAME}"
	log_progress_msg "Stop litecoind"
	do_stop
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  reload)
	log_daemon_msg "${NAME}"
	log_progress_msg "Reload litecoind"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  restart)
	log_daemon_msg "${NAME}"
	log_progress_msg "Restart litecoind"
	do_reload
	RESULT=$?
	case ${RESULT} in
	    0) log_end_msg 0 ;;
	    1) log_end_msg 0 ;;
	    2) log_end_msg 1 ;;
	    *) log_end_msg 1 ;;
	esac
	;;
  *)
	/bin/echo "Usage: $SCRIPTNAME {start|stop|restart|reload|start-reindex|start-rescan|start-zapwallettxes|start-upgradewallet}" >&2
	exit 3
	;;
esac

exit ${RESULT}
